@@ -14,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import with_statement, division
+from __future__ import division
 
 import re
 import os
@@ -76,7 +76,9 @@ class TestWMTS(SystemTest):
         eq_(resp.content_type, 'application/xml')
         xml = resp.lxml
         assert validate_with_xsd(xml, xsd_name='wmts/1.0/wmtsGetCapabilities_response.xsd')
-        eq_(len(xml.xpath('//wmts:Layer', namespaces=ns_wmts)), 5)
+        eq_(xml.xpath('//wmts:Layer/ows:Identifier/text()', namespaces=ns_wmts),
+            ['wms_cache','wms_cache_multi','tms_cache','tms_cache_ul','gk3_cache'],
+        )
         eq_(len(xml.xpath('//wmts:Contents/wmts:TileMatrixSet', namespaces=ns_wmts)), 5)
 
         goog_matrixset = xml.xpath('//wmts:Contents/wmts:TileMatrixSet[./ows:Identifier/text()="GoogleMapsCompatible"]', namespaces=ns_wmts)[0]
@@ -95,6 +97,13 @@ class TestWMTS(SystemTest):
         data = BytesIO(resp.body)
         assert is_jpeg(data)
 
+        # test with integer tilematrix
+        url = str(self.common_tile_req).replace('=01', '=1')
+        resp = self.app.get(url)
+        eq_(resp.content_type, 'image/jpeg')
+        data = BytesIO(resp.body)
+        assert is_jpeg(data)
+
     def test_get_tile_flipped_axis(self):
         # test default tile lock directory
         tiles_lock_dir = os.path.join(test_config['base_dir'], 'cache_data', 'tile_locks')
