@@ -14,21 +14,28 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import with_statement
 
 import os
 from io import BytesIO
 from mapproxy.compat.image import Image, ImageDraw
-from mapproxy.image import ImageSource, ReadBufWrapper, is_single_color_image
-from mapproxy.image import peek_image_format
-from mapproxy.image.merge import merge_images
-from mapproxy.image import _make_transparent as make_transparent, SubImageSource, img_has_transparency, quantize
+from mapproxy.image import (
+    ImageSource,
+    BlankImageSource,
+    ReadBufWrapper,
+    is_single_color_image,
+    peek_image_format,
+    _make_transparent as make_transparent,
+    SubImageSource,
+    img_has_transparency,
+    quantize,
+)
+from mapproxy.image.merge import merge_images, BandMerger
 from mapproxy.image.opts import ImageOptions
 from mapproxy.image.tile import TileMerger, TileSplitter
-from mapproxy.image.transform import ImageTransformer
+from mapproxy.image.transform import ImageTransformer, transform_meshes
 from mapproxy.test.image import is_png, is_jpeg, is_tiff, create_tmp_image_file, check_format, create_debug_img, create_image
 from mapproxy.srs import SRS
-from nose.tools import eq_
+from nose.tools import eq_, assert_almost_equal
 from mapproxy.test.image import assert_img_colors_eq
 from nose.plugins.skip import SkipTest
 
@@ -90,6 +97,21 @@ class TestImageSource(object):
         assert is_tiff(ir.as_buffer(TIFF_FORMAT))
         assert is_tiff(ir.as_buffer())
 
+
+    def test_output_formats_greyscale_png(self):
+        img = Image.new('L', (100, 100))
+        ir = ImageSource(img, image_opts=PNG_FORMAT)
+        img = Image.open(ir.as_buffer(ImageOptions(colors=256, transparent=True, format='image/png')))
+        assert img.mode == 'P'
+        assert img.getpixel((0, 0)) == 255
+
+    def test_output_formats_greyscale_alpha_png(self):
+        img = Image.new('LA', (100, 100))
+        ir = ImageSource(img, image_opts=PNG_FORMAT)
+        img = Image.open(ir.as_buffer(ImageOptions(colors=256, transparent=True, format='image/png')))
+        assert img.mode == 'LA'
+        assert img.getpixel((0, 0)) == (0, 0)
+
     def test_output_formats_png8(self):
         img = Image.new('RGBA', (100, 100))
         ir = ImageSource(img, image_opts=PNG_FORMAT)
@@ -106,6 +128,17 @@ class TestImageSource(object):
         eq_(img.mode, 'RGBA')
         assert img.getpixel((0, 0)) == (0, 0, 0, 0)
 
+    def test_save_with_unsupported_transparency(self):
+        # check if encoding of non-RGB image with tuple as transparency
+        # works. workaround for Pillow #2633
+        img = Image.new('P', (100, 100))
+        img.info['transparency'] = (0, 0, 0)
+        image_opts = PNG_FORMAT.copy()
+
+        ir = ImageSource(img, image_opts=image_opts)
+        img = Image.open(ir.as_buffer())
+        eq_(img.mode, 'P')
+
 class TestSubImageSource(object):
     def test_full(self):
         sub_img = create_image((100, 100), color=[100, 120, 130, 140])
@@ -296,6 +329,17 @@ class TestLayerMerge(object):
             (10*10, (127, 127, 255, 255)),
         ])
 
+    def test_merge_L(self):
+        img1 = ImageSource(Image.new('RGBA', (10, 10), (255, 0, 255, 255)))
+        img2 = ImageSource(Image.new('L', (10, 10), 100))
+
+        # img2 overlays img1
+        result = merge_images([img1, img2], ImageOptions(transparent=True))
+        img = result.as_image()
+        assert_img_colors_eq(img, [
+            (10*10, (100, 100, 100, 255)),
+        ])
+
     def test_paletted_merge(self):
         if not hasattr(Image, 'FASTOCTREE'):
             raise SkipTest()
@@ -332,6 +376,16 @@ class TestLayerMerge(object):
         img = result.as_image()
         eq_(img.getpixel((0, 0)), (0, 255, 255))
 
+    def test_merge_rgb_with_transp(self):
+        img1 = ImageSource(Image.new('RGB', (10, 10), (255, 0, 255)))
+        raw = Image.new('RGB', (10, 10), (0, 255, 255))
+        raw.info = {'transparency': (0, 255, 255)} # make full transparent
+        img2 = ImageSource(raw)
+
+        result = merge_images([img1, img2], ImageOptions(transparent=False))
+        img = result.as_image()
+        eq_(img.getpixel((0, 0)), (255, 0, 255))
+
 
 class TestLayerCompositeMerge(object):
     def test_composite_merge(self):
@@ -389,23 +443,84 @@ class TestTransform(object):
         self.dst_srs = SRS(4326)
         self.dst_bbox = (0.2, 45.1, 8.3, 53.2)
         self.src_bbox = self.dst_srs.transform_bbox_to(self.src_srs, self.dst_bbox)
-    def test_transform(self, mesh_div=4):
-        transformer = ImageTransformer(self.src_srs, self.dst_srs, mesh_div=mesh_div)
+    def test_transform(self):
+        transformer = ImageTransformer(self.src_srs, self.dst_srs)
         result = transformer.transform(self.src_img, self.src_bbox, self.dst_size, self.dst_bbox,
             image_opts=ImageOptions(resampling='nearest'))
         assert isinstance(result, ImageSource)
-        assert result.as_image() != self.src_img
+        assert result.as_image() != self.src_img.as_image()
         assert result.size == (100, 150)
 
-    def _test_compare_mesh_div(self):
+    def _test_compare_max_px_err(self):
         """
         Create transformations with different div values.
         """
-        for div in [1, 2, 4, 6, 8, 12, 16]:
-            transformer = ImageTransformer(self.src_srs, self.dst_srs, mesh_div=div)
+        for err in [0.2, 0.5, 1, 2, 4, 6, 8, 12, 16]:
+            transformer = ImageTransformer(self.src_srs, self.dst_srs, max_px_err=err)
             result = transformer.transform(self.src_img, self.src_bbox,
-                                           self.dst_size, self.dst_bbox)
-            result.as_image().save('/tmp/transform-%d.png' % (div,))
+                                           self.dst_size, self.dst_bbox,
+                                           image_opts=ImageOptions(resampling='nearest'))
+            result.as_image().save('/tmp/transform-%03d.png' % (err*10,))
+
+
+class TestMesh(object):
+
+    def test_mesh_utm(self):
+        meshes = transform_meshes(
+            src_size=(1335, 1531),
+            src_bbox=(3.65, 39.84, 17.00, 55.15),
+            src_srs=SRS(4326),
+            dst_size=(853, 1683),
+            dst_bbox=(158512, 4428236, 1012321, 6111268),
+            dst_srs=SRS(25832),
+        )
+        eq_(len(meshes), 40)
+
+    def test_mesh_none(self):
+        meshes = transform_meshes(
+            src_size=(1000, 1500),
+            src_bbox=(0, 0, 10, 15),
+            src_srs=SRS(4326),
+            dst_size=(1000, 1500),
+            dst_bbox=(0, 0, 10, 15),
+            dst_srs=SRS(4326),
+        )
+
+        eq_(meshes, [((0, 0, 1000, 1500), [0.0, 0.0, 0.0, 1500.0, 1000.0, 1500.0, 1000.0, 0.0])])
+        eq_(len(meshes), 1)
+
+
+    def test_mesh(self):
+        # low map scale -> more meshes
+        # print(SRS(4326).transform_bbox_to(SRS(3857), (5, 50, 10, 55)))
+        meshes = transform_meshes(
+            src_size=(1000, 2000),
+            src_bbox=(556597, 6446275, 1113194, 7361866),
+            src_srs=SRS(3857),
+            dst_size=(1000, 1000),
+            dst_bbox=(5, 50, 10, 55),
+            dst_srs=SRS(4326),
+        )
+        eq_(len(meshes), 16)
+
+        # large map scale -> one meshes
+        # print(SRS(4326).transform_bbox_to(SRS(3857), (5, 50, 5.1, 50.1)))
+        meshes = transform_meshes(
+            src_size=(1000, 2000),
+            src_bbox=(556597.4539663672, 6446275.841017158,
+                      567729.4030456939, 6463612.124257667),
+            src_srs=SRS(3857),
+            dst_size=(1000, 1000),
+            dst_bbox=(5, 50, 5.1, 50.1),
+            dst_srs=SRS(4326),
+        )
+        eq_(len(meshes), 1)
+
+        # quad stretches whole image plus 1 pixel
+        eq_(meshes[0][0], (0, 0, 1000, 1000))
+        for e, a in zip(meshes[0][1], [0.0, 0.0, 0.0, 2000.0, 1000.0, 2000.0, 1000.0, 0.0]):
+            assert_almost_equal(e, a)
+
 
 
 class TestSingleColorImage(object):
@@ -560,3 +675,195 @@ class TestPeekImageFormat(object):
         buf = BytesIO()
         Image.new('RGB', (100, 100)).save(buf, format)
         eq_(peek_image_format(buf), expected_format)
+
+class TestBandMerge(object):
+    def setup(self):
+        self.img0 = ImageSource(Image.new('RGB', (10, 10), (0, 10, 20)))
+        self.img1 = ImageSource(Image.new('RGB', (10, 10), (100, 110, 120)))
+        self.img2 = ImageSource(Image.new('RGB', (10, 10), (200, 210, 220)))
+        self.img3 = ImageSource(Image.new('RGB', (10, 10), (0, 255, 0)))
+        self.blank = BlankImageSource(size=(10, 10), image_opts=ImageOptions())
+
+    def test_merge_noops(self):
+        """
+        Check that black image is returned for no ops.
+        """
+        merger = BandMerger(mode='RGB')
+
+        img_opts = ImageOptions('RGB')
+        result = merger.merge([self.img0], img_opts)
+        img = result.as_image()
+        eq_(img.size, (10, 10))
+        eq_(img.getpixel((0, 0)), (0, 0, 0))
+
+    def test_merge_missing_source(self):
+        """
+        Check that empty source list or source list with missing images
+        returns BlankImageSource.
+        """
+        merger = BandMerger(mode='RGB')
+        merger.add_ops(dst_band=0, src_img=0, src_band=0)
+        merger.add_ops(dst_band=1, src_img=1, src_band=0)
+        merger.add_ops(dst_band=2, src_img=2, src_band=0)
+
+        img_opts = ImageOptions('RGBA', transparent=True)
+        result = merger.merge([], img_opts, size=(10, 10))
+        img = result.as_image()
+
+        eq_(img.size, (10, 10))
+        eq_(img.getpixel((0, 0)), (255, 255, 255, 0))
+
+        result = merger.merge([self.img0, self.img1], img_opts, size=(10, 10))
+        img = result.as_image()
+
+        eq_(img.size, (10, 10))
+        eq_(img.getpixel((0, 0)), (255, 255, 255, 0))
+
+
+    def test_rgb_merge(self):
+        """
+        Check merge of RGB bands
+        """
+        merger = BandMerger(mode='RGB')
+
+        merger.add_ops(dst_band=1, src_img=0, src_band=0, factor=0.5)
+        merger.add_ops(dst_band=1, src_img=3, src_band=1, factor=0.5)
+        merger.add_ops(dst_band=0, src_img=2, src_band=1)
+        merger.add_ops(dst_band=2, src_img=1, src_band=2)
+
+        img_opts = ImageOptions('RGB')
+        result = merger.merge([self.img0, self.img1, self.img2, self.img3], img_opts)
+        img = result.as_image()
+
+        eq_(img.getpixel((0, 0)), (210, 127, 120))
+
+    def test_rgb_merge_missing(self):
+        """
+        Check missing band is set to 0
+        """
+        merger = BandMerger(mode='RGB')
+
+        merger.add_ops(dst_band=0, src_img=2, src_band=1)
+        merger.add_ops(dst_band=2, src_img=1, src_band=2)
+
+        img_opts = ImageOptions('RGB')
+        result = merger.merge([self.img0, self.img1, self.img2, self.img3], img_opts)
+        img = result.as_image()
+
+        eq_(img.getpixel((0, 0)), (210, 0, 120))
+
+    def test_rgba_merge(self):
+        """
+        Check merge of RGBA bands
+        """
+        merger = BandMerger(mode='RGBA')
+
+        merger.add_ops(dst_band=1, src_img=0, src_band=0, factor=0.5)
+        merger.add_ops(dst_band=1, src_img=3, src_band=1, factor=0.5)
+        merger.add_ops(dst_band=0, src_img=2, src_band=1)
+        merger.add_ops(dst_band=2, src_img=1, src_band=2)
+        merger.add_ops(dst_band=3, src_img=1, src_band=1)
+
+        img_opts = ImageOptions('RGBA')
+        result = merger.merge([self.img0, self.img1, self.img2, self.img3], img_opts)
+        img = result.as_image()
+
+        eq_(img.getpixel((0, 0)), (210, 127, 120, 110))
+
+    def test_rgba_merge_missing_a(self):
+        """
+        Check that missing alpha band defaults to opaque
+        """
+        merger = BandMerger(mode='RGBA')
+
+        merger.add_ops(dst_band=1, src_img=0, src_band=0, factor=0.5)
+        merger.add_ops(dst_band=1, src_img=3, src_band=1, factor=0.5)
+        merger.add_ops(dst_band=0, src_img=2, src_band=1)
+        merger.add_ops(dst_band=2, src_img=1, src_band=2)
+
+        img_opts = ImageOptions('RGBA')
+        result = merger.merge([self.img0, self.img1, self.img2, self.img3], img_opts)
+        img = result.as_image()
+
+        eq_(img.getpixel((0, 0)), (210, 127, 120, 255))
+
+    def test_l_merge(self):
+        """
+        Check merge bands to grayscale image
+        """
+        merger = BandMerger(mode='L')
+
+        merger.add_ops(dst_band=0, src_img=0, src_band=2, factor=0.2)
+        merger.add_ops(dst_band=0, src_img=2, src_band=1, factor=0.3)
+        merger.add_ops(dst_band=0, src_img=3, src_band=1, factor=0.5)
+
+        img_opts = ImageOptions('L')
+        result = merger.merge([self.img0, self.img1, self.img2, self.img3], img_opts)
+        img = result.as_image()
+
+        eq_(img.getpixel((0, 0)), int(20*0.2) + int(210*0.3) + int(255*0.5))
+
+    def test_p_merge(self):
+        """
+        Check merge bands to paletted image
+        """
+        merger = BandMerger(mode='RGB')
+
+        merger.add_ops(dst_band=1, src_img=0, src_band=0, factor=0.5)
+        merger.add_ops(dst_band=1, src_img=3, src_band=1, factor=0.5)
+        merger.add_ops(dst_band=0, src_img=2, src_band=1)
+        merger.add_ops(dst_band=2, src_img=1, src_band=2)
+
+        img_opts = ImageOptions('P', format='image/png', encoding_options={'quantizer': 'mediancut'})
+        result = merger.merge([self.img0, self.img1, self.img2, self.img3], img_opts)
+
+        # need to encode to get conversion to P
+        img = Image.open(result.as_buffer())
+
+        eq_(img.mode, 'P')
+        img = img.convert('RGB')
+        eq_(img.getpixel((0, 0)), (210, 127, 120))
+
+    def test_from_p_merge(self):
+        """
+        Check merge bands from paletted image
+        """
+        merger = BandMerger(mode='RGB')
+
+        merger.add_ops(dst_band=0, src_img=0, src_band=2)
+        merger.add_ops(dst_band=1, src_img=0, src_band=1)
+        merger.add_ops(dst_band=2, src_img=0, src_band=0)
+
+        img = Image.new('RGB', (10, 10), (0, 100, 200)).quantize(256)
+        eq_(img.mode, 'P')
+        # src img is P but we can still access RGB bands
+        src_img = ImageSource(img)
+
+        img_opts = ImageOptions('RGB')
+        result = merger.merge([src_img], img_opts)
+
+        img = result.as_image()
+        eq_(img.mode, 'RGB')
+        eq_(img.getpixel((0, 0)), (200, 100, 0))
+
+    def test_from_mixed_merge(self):
+        """
+        Check merge RGBA bands from image without alpha (mixed)
+        """
+        merger = BandMerger(mode='RGBA')
+
+        merger.add_ops(dst_band=0, src_img=0, src_band=2)
+        merger.add_ops(dst_band=1, src_img=0, src_band=1)
+        merger.add_ops(dst_band=2, src_img=0, src_band=0)
+        merger.add_ops(dst_band=3, src_img=0, src_band=3)
+
+        img = Image.new('RGB', (10, 10), (0, 100, 200))
+        src_img = ImageSource(img)
+
+        img_opts = ImageOptions('RGBA')
+        result = merger.merge([src_img], img_opts)
+
+        img = result.as_image()
+        eq_(img.mode, 'RGBA')
+        eq_(img.getpixel((0, 0)), (200, 100, 0, 255))
+
